@@ -1,3 +1,5 @@
+from typing_extensions import TypeAlias
+
 from ._types import (
     TextEvent as TextEvent,
     InputJsonEvent as InputJsonEvent,
@@ -12,12 +14,20 @@ from ._messages import (
     AsyncMessageStreamManager as AsyncMessageStreamManager,
 )
 from ._beta_types import (
-    BetaTextEvent as BetaTextEvent,
     BetaInputJsonEvent as BetaInputJsonEvent,
-    BetaMessageStopEvent as BetaMessageStopEvent,
-    BetaMessageStreamEvent as BetaMessageStreamEvent,
-    BetaContentBlockStopEvent as BetaContentBlockStopEvent,
+    ParsedBetaTextEvent as ParsedBetaTextEvent,
+    ParsedBetaMessageStopEvent as ParsedBetaMessageStopEvent,
+    ParsedBetaMessageStreamEvent as ParsedBetaMessageStreamEvent,
+    ParsedBetaContentBlockStopEvent as ParsedBetaContentBlockStopEvent,
 )
+
+# For backwards compatibility
+BetaTextEvent: TypeAlias = ParsedBetaTextEvent
+BetaMessageStopEvent: TypeAlias = ParsedBetaMessageStopEvent[object]
+BetaMessageStreamEvent: TypeAlias = ParsedBetaMessageStreamEvent
+BetaContentBlockStopEvent: TypeAlias = ParsedBetaContentBlockStopEvent[object]
+
+
 from ._beta_messages import (
     BetaMessageStream as BetaMessageStream,
     BetaAsyncMessageStream as BetaAsyncMessageStream,
